 package Model;

import java.util.ArrayList;

import org.eclipse.elk.graph.ElkEdge;

/**
 * Ein Interface, welches die Methoden einer Kante aus einem gelayerten Graphen beschreibt
 * @author kolja
 *
 */
public interface LayeredGraphEdge {

    /**
     * @return Gibt die originale Kante des Elk Graphen zurck
     */
    ElkEdge getOriginalEdge();
    
    /**
     * Entfernt diese Kante vom Graphen
     */
    void remove();
    /**
     * Gibt eine Liste mit Knoten zurck, von welchen die Kante beginnt
     * @return Liste mit Startknoten
     */
    ArrayList< LayeredGraphNode > getSources();
    /**
     * Gibt eine Liste mit Knoten zurck, bei welchen die Kante endet
     * @return Liste mit Endknoten
     */
    ArrayList< LayeredGraphNode > getTargets();
    /**
     * Prft, ob die Kante ber mehrere Layer hinweg geht
     * @return true, falls die Kante ber mehrere Layer hinweg geht
     */
    boolean isCrossLayerEdge();
    /**
     * Fr jeden Layer ber den die Kante geht wird ein Dummy Node erzeugt.
     * Diese werden durch Dummy Kanten verbunden, welche diese Kante ersetzen.
     * Die Kante wird automatisch aus dem Graphen entfernt
     */
    void replaceByDummyNodes();
    /**
     * Dreht die Kante um, indem die Startknoten mit den Endknoten getauscht werden
     */
    void reverse();
    /**
     * Prft, ob die Kante umgedreht wurde
     * @return true, falls die Kante umgedreht wurde
     */
    boolean isReversedEdge();
    /**
     * Prft, ob die Kante eine Dummy Kante ist (also ob sie von einem Dummy Knoten startet oder zu einem DummyKnoten hinfhrt)
     * @return true, falls die Kante eine Dummy Kante ist
     */
    boolean isDummyEdge();
    /**
     * Ermittelt, welche fr welche originale Kante diese Kante als Dummykante eingesetzt wurd
     * und entfernt all die Dummynodes und Dummykanten wieder. Anschlieend wird die alte Kante wieder zum Graphen hinzugefgt
     */
    void removeDummyNodes();
    /**
     * Legt fest, dass es sich bei dieser Kante um eine Dummykante handelt
     */
    void setDummyEdge();
    /**
     * Legt fest, dass es sich bei dieser Kante um eine umgedrehte Kante handelt
     */
    void setReversedEdge();
    /**
     * Legt den Graphen fest, zu dem diese Kante gehrt
     * @param graph Der Graph, zu dem die Kante gehrt
     */
    void setGraph( LayeredGraphNode graph );
}
